import {
  IsOptional,
  IsBoolean,
  IsInt,
  IsEnum,
  ValidateNested,
  ArrayMinSize,
  IsUUID,
  IsArray,
  IsISO8601,
} from 'class-validator';
import { Type } from 'class-transformer';
import { ScheduleItemDto } from './schedule-item.dto';
import { WaypointDto } from './waypoint.dto';
import { HasValidPositionIndexes } from './validators/decorators/has-valid-position-indexes.decorator';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { IsAfterOrEqual } from './validators/decorators/is-after-or-equal.decorator';
import { HasRole } from './validators/decorators/has-role.decorator';
import { HasSeats } from './validators/decorators/has-seats.decorator';

export class CreateAdRequestDto {
  @IsUUID(4)
  userId: string;

  @IsBoolean()
  @HasRole('passenger', {
    message: 'At least one of driver or passenger property needs to be truthy',
  })
  @HasSeats('seatsProposed', {
    message: 'Number of seats proposed as a driver is required',
  })
  driver: boolean;

  @IsBoolean()
  @HasSeats('seatsRequested', {
    message: 'Number of seats requested as a passenger is required',
  })
  passenger: boolean;

  @IsEnum(Frequency)
  frequency: Frequency;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  fromDate: string;

  @IsISO8601({
    strict: true,
    strictSeparator: true,
  })
  @IsAfterOrEqual('fromDate', {
    message: 'toDate must be after or equal to fromDate',
  })
  toDate: string;

  @Type(() => ScheduleItemDto)
  @IsArray()
  @ArrayMinSize(1)
  @ValidateNested({ each: true })
  schedule: ScheduleItemDto[];

  @IsInt()
  @IsOptional()
  seatsProposed?: number;

  @IsInt()
  @IsOptional()
  seatsRequested?: number;

  @IsBoolean()
  strict: boolean;

  @Type(() => WaypointDto)
  @IsArray()
  @ArrayMinSize(2)
  @HasValidPositionIndexes()
  @ValidateNested({ each: true })
  waypoints: WaypointDto[];
}
